/*-
 * Copyright (c) 2003 Sren Schmidt <sos@FreeBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $FreeBSD$
 */

#include <config.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/ata.h>

#include "ScsiIf.h"
#include "util.h"

class ScsiIfImpl {
public:
	char *			devname;
	int			ctrldev;
	int			channel;
	int			device;
	char			ccb[16];
	int			timeout;
};

ScsiIf::ScsiIf(const char *devname)
{
	impl_ = new ScsiIfImpl;
	impl_->devname = strdupCC(devname);
	impl_->timeout = 5000;

	maxDataLen_ = 32 * 1024;
	vendor_[0] = 0;
	product_[0] = 0;
	revision_[0] = 0;
}

ScsiIf::~ScsiIf()
{
	delete[] impl_->devname;
	delete impl_;
}

// opens device
// return: 0: OK
//         1: device could not be opened
//         2: inquiry failed
int ScsiIf::init()
{
	struct ata_cmd iocmd;
	int maxunit, channel;

        if ((impl_->ctrldev = open("/dev/ata", O_RDWR)) < 0) {
                message(-2, "Cannot open /dev/ata for control");
		return 1;
	}
  
        bzero(&iocmd, sizeof(struct ata_cmd));
	iocmd.cmd = ATAGMAXCHANNEL;
	if (ioctl(impl_->ctrldev, IOCATA, &iocmd) < 0) {
		message(-2, "Cannot get max ATA channel count");
		return 1;
	}
	maxunit = iocmd.u.maxchan;

	for (channel = 0; channel < maxunit; channel++) {
        	bzero(&iocmd, sizeof(struct ata_cmd));
        	iocmd.channel = channel;
        	iocmd.device = -1;
        	iocmd.cmd = ATAGPARM;
        	if (ioctl(impl_->ctrldev, IOCATA, &iocmd) < 0)
			return 1;
		
		if (iocmd.u.param.type[0] &&
		    !strcmp(impl_->devname, iocmd.u.param.name[0])) {
			strncpy(vendor_,
			    (const char *)&iocmd.u.param.params[0].model, 8);
			vendor_[8] = 0;
			strncpy(product_,
			    (const char *)&iocmd.u.param.params[0].model+8, 16);
			product_[16] = 0;
			strncpy(revision_,
			    (const char *)&iocmd.u.param.params[0].revision,4);
			revision_[4] = 0;
			impl_->channel = channel;
			impl_->device = 0;
			return 0;
		}
		if (iocmd.u.param.type[1] &&
		    !strcmp(impl_->devname, iocmd.u.param.name[1])) {
			strncpy(vendor_,
			    (const char *)&iocmd.u.param.params[1].model, 8);
			vendor_[8] = 0;
			strncpy(product_,
			    (const char *)&iocmd.u.param.params[1].model+8, 16);
			product_[16] = 0;
			strncpy(revision_,
			    (const char *)&iocmd.u.param.params[1].revision,4);
			revision_[4] = 0;
			impl_->channel = channel;
			impl_->device = 1;
			return 0;
		}
	}
	return 2;
}

// Set given timeout value in seconds and returns old timeout.
int ScsiIf::timeout(int t)
{
	int old = impl_->timeout;
	impl_->timeout = t;
	return old;
}

// sends command and receives data
// return 0: OK
//        1: scsi command failed (os level, no sense data available)
//        2: scsi command failed (sense data available)
int ScsiIf::sendCmd(const unsigned char *cmd, int cmdLen, 
		    const unsigned char *dataOut, int dataOutLen,
		    unsigned char *dataIn, int dataInLen,
		    int showMessage)
{
	struct ata_cmd iocmd;

        bzero(&iocmd, sizeof(struct ata_cmd));
	bcopy(cmd, &iocmd.u.request.u.atapi.ccb, cmdLen);
	if (dataOut && dataOutLen > 0) {
		iocmd.u.request.data = (caddr_t)dataOut;
		iocmd.u.request.count = dataOutLen;
		iocmd.u.request.flags = (ATA_CMD_ATAPI | ATA_CMD_WRITE);
	}
	else if (dataIn && dataInLen > 0) {
		iocmd.u.request.data = (caddr_t)dataIn;
		iocmd.u.request.count = dataInLen;
		iocmd.u.request.flags = (ATA_CMD_ATAPI | ATA_CMD_READ);
	}
	else {
		iocmd.u.request.flags = (ATA_CMD_ATAPI | ATA_CMD_CONTROL);
	}
	iocmd.u.request.timeout = impl_->timeout;
       	iocmd.channel = impl_->channel;
       	iocmd.device = impl_->device;
	iocmd.cmd = ATAREQUEST;
       	if (ioctl(impl_->ctrldev, IOCATA, &iocmd) < 0)
		return 1;
	if (iocmd.u.request.error)
		return 1;
	return 0;
}

const unsigned char *ScsiIf::getSense(int &len) const
{
	return NULL;
}

ScsiIf::ScanData *ScsiIf::scan(int *len)
{
	*len = 0;
	return NULL;
}

void ScsiIf::printError()
{
}

#include "ScsiIf-common.cc"
